----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:12:45 05/02/2013 
-- Design Name: 
-- Module Name:    ADC_buffer - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;
use work.DOSI_pack.all;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity ADC_buffer is
    Port ( clk_div : in  STD_LOGIC;
           DIin : in  STD_LOGIC_VECTOR (95 downto 0);
           DIDin : in  STD_LOGIC_VECTOR (95 downto 0);
           DQin : in  STD_LOGIC_VECTOR (95 downto 0);
           DQDin : in  STD_LOGIC_VECTOR (95 downto 0);
           reset : in  STD_LOGIC;
           sample : in  STD_LOGIC;
           train : in  STD_LOGIC;
           train_done : out  STD_LOGIC;
					 delay : in array48X4;
           train_waddr : out  STD_LOGIC_VECTOR (15 downto 0);
           ipb_clk : in  STD_LOGIC;
           ipb_addr : in  STD_LOGIC_VECTOR (31 downto 0);
           ipb_strobe : in  STD_LOGIC;
           ipb_write : in  STD_LOGIC;
           ipb_rdata : out  STD_LOGIC_VECTOR (31 downto 0));
end ADC_buffer;

architecture Behavioral of ADC_buffer is
signal waddr : std_logic_vector(11 downto 0) := (others => '0');
signal raddr : std_logic_vector(13 downto 0) := (others => '0');
signal din : std_logic_vector(383 downto 0) := (others => '0');
signal din_dl : std_logic_vector(383 downto 0) := (others => '0');
signal din_dl_q : std_logic_vector(383 downto 0) := (others => '0');
signal buf_di : std_logic_vector(383 downto 0) := (others => '0');
signal buf_DoA : std_logic_vector(47 downto 0) := (others => '0');
signal buf_DoB : std_logic_vector(47 downto 0) := (others => '0');
signal WE_A : std_logic_vector(0 downto 0) := (others => '0');
signal WE_B : std_logic_vector(0 downto 0) := (others => '0');
signal RST_A : std_logic := '0';
signal RST_B : std_logic := '0';
signal WrEn : std_logic := '0';
signal training : std_logic := '0';
signal got_pattern : std_logic := '0';
signal pattern : std_logic_vector(7 downto 0) := (others => '0');
signal pattern_sel : std_logic_vector(2 downto 0) := (others => '0');
signal pattern_cntr : std_logic_vector(4 downto 0) := (others => '0');
signal train_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal sampleSyncReg : std_logic_vector(3 downto 0) := (others => '0');
signal trainSyncReg : std_logic_vector(3 downto 0) := (others => '0');

begin
din <= DQDin & DQin & DIDin & DIin;
g_din_dl: for i in 0 to 383 generate
   i_din_dl : SRL16E
   port map (
      Q => din_dl(i),       -- SRL data output
      A0 => delay(i/8)(0),     -- Select[0] input
      A1 => delay(i/8)(1),     -- Select[1] input
      A2 => delay(i/8)(2),     -- Select[2] input
      A3 => delay(i/8)(3),     -- Select[3] input
      CE => '1',     -- Clock enable input
      CLK => clk_div,   -- Clock input
      D => din(i)        -- SRL data input
   );
end generate;
process(clk_div)
begin
	if(clk_div'event and clk_div = '1')then
		for i in 0 to 11 loop
			din_dl_q(i*8+7 downto i*8) <=  din_dl(84+i) & din_dl(72+i) & din_dl(60+i) & din_dl(48+i) & din_dl(36+i) & din_dl(24+i) & din_dl(12+i) & din_dl(i);
			din_dl_q(i*8+103 downto i*8+96) <= din_dl(180+i) & din_dl(168+i) & din_dl(156+i) & din_dl(144+i) & din_dl(132+i) & din_dl(120+i) & din_dl(108+i) & din_dl(96+i);
			din_dl_q(i*8+199 downto i*8+192) <= din_dl(276+i) & din_dl(264+i) & din_dl(252+i) & din_dl(240+i) & din_dl(228+i) & din_dl(216+i) & din_dl(204+i) & din_dl(192+i);
			din_dl_q(i*8+295 downto i*8+288) <= din_dl(372+i) & din_dl(360+i) & din_dl(348+i) & din_dl(336+i) & din_dl(324+i) & din_dl(312+i) & din_dl(300+i) & din_dl(288+i);
		end loop;
		buf_di(63 downto 0) <= din_dl_q(63 downto 0);
		buf_di(71 downto 64) <= not din_dl_q(71 downto 64); -- DI8 polarity wrong
		buf_di(383 downto 72) <= din_dl_q(383 downto 72);
		if(reset = '1')then
			waddr <= (others => '0');
		elsif(WrEn = '1')then
			if(and_reduce(waddr(9 downto 0)) = '1')then
				waddr(11) <= waddr(10);
				waddr(10) <= not waddr(11) and not waddr(10);
			end if;
			waddr(9 downto 0) <= waddr(9 downto 0) + 1;
		end if;
		sampleSyncReg <= sampleSyncReg(2 downto 0) & sample;
		trainSyncReg <= trainSyncReg(2 downto 0) & train;
		if(reset = '1' or train_cntr(7) = '1' or (training = '0' and and_reduce(waddr(7 downto 0)) = '1'))then
			WrEn <= '0';
		elsif(sampleSyncReg(3 downto 2) = "01" or trainSyncReg(3 downto 2) = "01")then
			WrEn <= '1';
		end if;
		if(reset = '1' or train_cntr(7) = '1')then
			training <= '0';
		elsif(trainSyncReg(3 downto 2) = "01")then
			training <= '1';
		end if;
		if(train_cntr(7) = '1' and training = '1')then
			train_waddr <= x"0" & waddr;
		end if;
		if(reset = '1' or trainSyncReg(3 downto 2) = "01")then
			train_done <= '0';
		elsif(train_cntr(7) = '1')then
			train_done <= '1';
		end if;
		if(got_pattern = '0')then
			train_cntr <= (others => '0');
		else
			train_cntr <= train_cntr + 1;
		end if;
		if(reset = '1' or train_cntr(7) = '1')then
			got_pattern <= '0';
		elsif(pattern_cntr(4) = '1')then
			got_pattern <= '1';
		end if;
		case pattern_sel is
			when "000" => pattern <= x"4a";
			when "001" => pattern <= x"a5";
			when "010" => pattern <= x"52";
			when "011" => pattern <= x"29";
			when others => pattern <= x"94";
		end case;
		if(pattern /= buf_di(383 downto 376) or pattern_sel(2) = '1' or training = '0')then
			pattern_sel <= "000"; 
		else
			pattern_sel <= pattern_sel + 1;
		end if;
		if(training = '0' or pattern /= buf_di(383 downto 376))then
			pattern_cntr <= (others => '0');
		elsif(pattern_sel(2) = '1')then
			pattern_cntr <= pattern_cntr + 1;
		end if;
	end if;
end process;
process(buf_doA,buf_doB,ipb_addr)
variable s : std_logic_vector(1 downto 0);
begin
s := ipb_addr(16) & ipb_addr(0);
	case s is
		when "00" => ipb_rdata(11 downto 0) <= buf_doA(11 downto 0) or buf_doB(11 downto 0);
		when "01" => ipb_rdata(11 downto 0) <= buf_doA(23 downto 12) or buf_doB(23 downto 12);
		when "10" => ipb_rdata(11 downto 0) <= buf_doA(35 downto 24) or buf_doB(35 downto 24);
		when others => ipb_rdata(11 downto 0) <= buf_doA(47 downto 36) or buf_doB(47 downto 36);
	end case;
end process;
ipb_rdata(31 downto 12) <= (others => '0');
g_buf : for i in 0 to 47 generate
   i_bufA : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "18Kb", -- Target BRAM, "9Kb" or "18Kb" 
      DEVICE => "SPARTAN6", -- Target device: "VIRTEX5", "VIRTEX6", "SPARTAN6" 
      WRITE_WIDTH => 8,    -- Valid values are 1-36 
      READ_WIDTH => 1,     -- Valid values are 1-36
      DO_REG => 0, -- Optional output register (0 or 1)
      INIT_FILE => "NONE",
      SIM_COLLISION_CHECK => "NONE") -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
   port map (
      DO => buf_doA(i downto i),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => buf_di(i*8+7 downto i*8),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => raddr, -- Input read address, width defined by read port depth
      RDCLK => ipb_clk,   -- 1-bit input read clock
      RDEN => '1',     -- 1-bit input read port enable
      REGCE => '1',   -- 1-bit input read output register enable
      RST => RST_A,       -- 1-bit input reset 
      WE => WE_A,         -- Input write enable, width defined by write port depth
      WRADDR => waddr(10 downto 0), -- Input write address, width defined by write port depth
      WRCLK => clk_div,   -- 1-bit input write clock
      WREN => WrEn      -- 1-bit input write port enable
   );
   i_bufB : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "9Kb", -- Target BRAM, "9Kb" or "18Kb" 
      DEVICE => "SPARTAN6", -- Target device: "VIRTEX5", "VIRTEX6", "SPARTAN6" 
      WRITE_WIDTH => 8,    -- Valid values are 1-36 
      READ_WIDTH => 1,     -- Valid values are 1-36
      DO_REG => 0, -- Optional output register (0 or 1)
      INIT_FILE => "NONE",
      SIM_COLLISION_CHECK => "NONE") -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
   port map (
      DO => buf_doB(i downto i),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => buf_di(i*8+7 downto i*8),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => raddr(12 downto 0), -- Input read address, width defined by read port depth
      RDCLK => ipb_clk,   -- 1-bit input read clock
      RDEN => '1',     -- 1-bit input read port enable
      REGCE => '1',   -- 1-bit input read output register enable
      RST => RST_B,       -- 1-bit input reset 
      WE => WE_B,         -- Input write enable, width defined by write port depth
      WRADDR => waddr(9 downto 0), -- Input write address, width defined by write port depth
      WRCLK => clk_div,   -- 1-bit input write clock
      WREN => WrEn      -- 1-bit input write port enable
   );
end generate;
WE_A <= "1" when waddr(11) = '0' else "0";
WE_B <= "1" when waddr(11) = '1' else "0";
RST_A <= '0' when ipb_addr(15) = '0' else '1';
RST_B <= '0' when ipb_addr(15 downto 14) = "10" else '1';
raddr <= ipb_addr(14 downto 1);
end Behavioral;

