----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:16:05 04/30/2013 
-- Design Name: 
-- Module Name:    ADC_if - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity ADC_if is
    Port ( clk_p : in  STD_LOGIC;
           clk_n : in  STD_LOGIC;
           reset : in  STD_LOGIC; -- in rdclk domain, must be at least three clocks wide
           din_p : in  STD_LOGIC_VECTOR (11 downto 0);
           din_n : in  STD_LOGIC_VECTOR (11 downto 0);
           slip : in  STD_LOGIC;
           rdclk : in  STD_LOGIC;
           clkout : out  STD_LOGIC;
           dout : out  STD_LOGIC_VECTOR (95 downto 0));
end ADC_if;

architecture Behavioral of ADC_if is
component ADC_IN
generic
 (-- width of the data for the system
  sys_w       : integer := 12;
  -- width of the data for the device
  dev_w       : integer := 96);
port
 (
  -- From the system into the device
  DATA_IN_FROM_PINS_P     : in    std_logic_vector(sys_w-1 downto 0);
  DATA_IN_FROM_PINS_N     : in    std_logic_vector(sys_w-1 downto 0);
  DATA_IN_TO_DEVICE       : out   std_logic_vector(dev_w-1 downto 0);

  DEBUG_IN                : in    std_logic_vector (1 downto 0);       -- Input debug data. Tie to "00" if not used
  DEBUG_OUT               : out   std_logic_vector ((3*sys_w)+5 downto 0); -- Ouput debug data. Leave NC if not required
-- Clock and reset signals
  CLK_IN_P                : in    std_logic;                    -- Differential fast clock from IOB
  CLK_IN_N                : in    std_logic;
  CLK_DIV_OUT             : out   std_logic;                    -- Slow clock output
  IO_RESET                : in    std_logic);                   -- Reset signal for IO circuit
end component;
signal desdout : std_logic_vector(95 downto 0) := (others => '0');
signal buf_di : std_logic_vector(95 downto 0) := (others => '0');
signal buf_do : std_logic_vector(95 downto 0) := (others => '0');
signal waddr : std_logic_vector(3 downto 0) := (others => '0');
signal clk_div : std_logic := '0';
signal resetSyncReg : std_logic_vector(3 downto 0) := (others => '0');
signal raddr : std_logic_vector(3 downto 0) := (others => '0');
signal slipSyncReg : std_logic_vector(3 downto 0) := (others => '0');
signal slipOne : std_logic := '0';
begin
clkout <= clk_div;
i_ADC_IN : ADC_IN
  port map
   (
  -- From the system into the device
  DATA_IN_FROM_PINS_P =>   din_p, --Input pins
  DATA_IN_FROM_PINS_N =>   din_n, --Input pins
  DATA_IN_TO_DEVICE =>   desdout, --Output pins

  DEBUG_IN =>   "00", -- Tie to "00" if not used
  DEBUG_OUT =>   open, -- leave NC is not required
  
-- Clock and reset signals
  CLK_IN_P =>   clk_p,     -- Differential clock from IOB
  CLK_IN_N =>   clk_n,     -- Differential clock from IOB
  CLK_DIV_OUT =>   clk_div,     -- Slow clock output
  IO_RESET =>   reset);          --system reset
process(clk_div,reset)
begin
	if(reset = '1')then
		resetSyncReg <= (others => '1');
	elsif(clk_div'event and clk_div = '1')then
		resetSyncReg <= resetSyncReg(2 downto 0) & '0';
	end if;
end process;
process(clk_div)
begin
	if(clk_div'event and clk_div = '1')then
-- deserializer output format is ADC3 & ADC2 & ADC1 & ADC0;
-- rearrange it as ADC3(11) & ADC2(11) & ADC1(11) & ADC0(11) & ADC3(10) & ... & ADC1(0) & ADC0(0);
		buf_di <= desdout;
		for i in 0 to 11 loop
			buf_di(i*8+7 downto i*8) <= desdout(84+i) & desdout(72+i) & desdout(60+i) & desdout(48+i) & desdout(36+i) & desdout(24+i) & desdout(12+i) & desdout(i);
		end loop;
		if(resetSyncReg(3) = '1')then
			waddr <= (others => '0');
		else
			waddr <= waddr + 1;
		end if;
	end if;
end process;
g_buf: for i in 0 to 95 generate
   i_buf : RAM32X1D
   port map (
      DPO => buf_do(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => waddr(0),       -- R/W address[0] input bit
      A1 => waddr(1),       -- R/W address[1] input bit
      A2 => waddr(2),       -- R/W address[2] input bit
      A3 => waddr(3),       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => buf_di(i),         -- Write 1-bit data input
      DPRA0 => raddr(0), -- Read-only address[0] input bit
      DPRA1 => raddr(1), -- Read-only address[1] input bit
      DPRA2 => raddr(2), -- Read-only address[2] input bit
      DPRA3 => raddr(3), -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => clk_div,   -- Write clock input
      WE => '1'        -- Write enable input
   );
end generate;
process(rdclk)
begin
	if(rdclk'event and rdclk = '1')then
		if(reset = '1')then
			slipSyncReg <= x"0";
			slipOne <= '0';
			raddr <= x"8";
		else
			slipSyncReg <= slipSyncReg(2 downto 0) & slip;
			slipOne <= not slipSyncReg(3) and slipSyncReg(2);
			if(slipOne = '0')then
				raddr <= raddr + 1;
			end if;
		end if;
		dout <= buf_do;
	end if;
end process;
end Behavioral;

